@@ -27,7 +27,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 __title__ = 'Apprise'
-__version__ = '1.7.4'
+__version__ = '1.9.0'
 __author__ = 'Chris Caron'
 __license__ = 'BSD'
 __copywrite__ = 'Copyright (C) 2024 Chris Caron <lead2gold@gmail.com>'
@@ -48,18 +48,25 @@ from .common import ContentIncludeMode
 from .common import CONTENT_INCLUDE_MODES
 from .common import ContentLocation
 from .common import CONTENT_LOCATIONS
+from .common import PersistentStoreMode
+from .common import PERSISTENT_STORE_MODES
 
-from .URLBase import URLBase
-from .URLBase import PrivacyMode
-from .plugins.NotifyBase import NotifyBase
-from .config.ConfigBase import ConfigBase
-from .attachment.AttachBase import AttachBase
-
-from .Apprise import Apprise
-from .AppriseAsset import AppriseAsset
-from .AppriseConfig import AppriseConfig
-from .AppriseAttachment import AppriseAttachment
+from .url import URLBase
+from .url import PrivacyMode
+from .plugins.base import NotifyBase
+from .config.base import ConfigBase
+from .attachment.base import AttachBase
+from . import exception
 
+from .apprise import Apprise
+from .locale import AppriseLocale
+from .asset import AppriseAsset
+from .persistent_store import PersistentStore
+from .apprise_config import AppriseConfig
+from .apprise_attachment import AppriseAttachment
+from .manager_attachment import AttachmentManager
+from .manager_config import ConfigurationManager
+from .manager_plugins import NotificationManager
 from . import decorators
 
 # Inherit our logging with our additional entries added to it
@@ -73,7 +80,11 @@ logging.getLogger(__name__).addHandler(logging.NullHandler())
 __all__ = [
     # Core
     'Apprise', 'AppriseAsset', 'AppriseConfig', 'AppriseAttachment', 'URLBase',
-    'NotifyBase', 'ConfigBase', 'AttachBase',
+    'NotifyBase', 'ConfigBase', 'AttachBase', 'AppriseLocale',
+    'PersistentStore',
+
+    # Exceptions
+    'exception',
 
     # Reference
     'NotifyType', 'NotifyImageSize', 'NotifyFormat', 'OverflowMode',
@@ -81,8 +92,12 @@ __all__ = [
     'ConfigFormat', 'CONFIG_FORMATS',
     'ContentIncludeMode', 'CONTENT_INCLUDE_MODES',
     'ContentLocation', 'CONTENT_LOCATIONS',
+    'PersistentStoreMode', 'PERSISTENT_STORE_MODES',
     'PrivacyMode',
 
+    # Managers
+    'NotificationManager', 'ConfigurationManager', 'AttachmentManager',
+
     # Decorator
     'decorators',
 
