@@ -36,6 +36,7 @@ import threading
 from .utils import import_module
 from .utils import Singleton
 from .utils import parse_list
+from .utils import path_decode
 from os.path import dirname
 from os.path import abspath
 from os.path import join
@@ -61,6 +62,9 @@ class PluginManager(metaclass=Singleton):
     # The module path to scan
     module_path = join(abspath(dirname(__file__)), _id)
 
+    # For filtering our result when scanning a module
+    module_filter_re = re.compile(r'^(?P<name>((?!_)[A-Za-z0-9]+))$')
+
     # thread safe loading
     _lock = threading.Lock()
 
@@ -177,7 +181,7 @@ class PluginManager(metaclass=Singleton):
             # The .py extension is optional as we support loading directories
             # too
             module_re = re.compile(
-                r'^(?P<name>' + self.fname_prefix + r'[a-z0-9]+)(\.py)?$',
+                r'^(?P<name>(?!base|_)[a-z0-9_]+)(\.py)?$',
                 re.I)
 
             t_start = time.time()
@@ -188,10 +192,6 @@ class PluginManager(metaclass=Singleton):
                     # keep going
                     continue
 
-                elif match.group('name') == f'{self.fname_prefix}Base':
-                    # keep going
-                    continue
-
                 # Store our notification/plugin name:
                 module_name = match.group('name')
                 module_pyname = '{}.{}'.format(module_name_prefix, module_name)
@@ -216,7 +216,47 @@ class PluginManager(metaclass=Singleton):
                         # logging found in import_module and not needed here
                         continue
 
-                if not hasattr(module, module_name):
+                module_class = None
+                for m_class in [obj for obj in dir(module)
+                                if self.module_filter_re.match(obj)]:
+                    # Get our plugin
+                    plugin = getattr(module, m_class)
+                    if not hasattr(plugin, 'app_id'):
+                        # Filter out non-notification modules
+                        logger.trace(
+                            "(%s.%s) import failed; no app_id defined in %s",
+                            self.name, m_class, os.path.join(module_path, f))
+                        continue
+
+                    # Add our plugin name to our module map
+                    self._module_map[module_name] = {
+                        'plugin': set([plugin]),
+                        'module': module,
+                        'path': '{}.{}'.format(
+                            module_name_prefix, module_name),
+                        'native': True,
+                    }
+
+                    fn = getattr(plugin, 'schemas', None)
+                    schemas = set([]) if not callable(fn) else fn(plugin)
+
+                    # map our schema to our plugin
+                    for schema in schemas:
+                        if schema in self._schema_map:
+                            logger.error(
+                                "{} schema ({}) mismatch detected - {} to {}"
+                                .format(self.name, schema, self._schema_map,
+                                        plugin))
+                            continue
+
+                        # Assign plugin
+                        self._schema_map[schema] = plugin
+
+                    # Store our class
+                    module_class = m_class
+                    break
+
+                if not module_class:
                     # Not a library we can load as it doesn't follow the simple
                     # rule that the class must bear the same name as the
                     # notification file itself.
@@ -226,38 +266,6 @@ class PluginManager(metaclass=Singleton):
                         self.name, module_name, os.path.join(module_path, f))
                     continue
 
-                # Get our plugin
-                plugin = getattr(module, module_name)
-                if not hasattr(plugin, 'app_id'):
-                    # Filter out non-notification modules
-                    logger.trace(
-                        "(%s) import failed; no app_id defined in %s",
-                        self.name, module_name, os.path.join(module_path, f))
-                    continue
-
-                # Add our plugin name to our module map
-                self._module_map[module_name] = {
-                    'plugin': set([plugin]),
-                    'module': module,
-                    'path': '{}.{}'.format(module_name_prefix, module_name),
-                    'native': True,
-                }
-
-                fn = getattr(plugin, 'schemas', None)
-                schemas = set([]) if not callable(fn) else fn(plugin)
-
-                # map our schema to our plugin
-                for schema in schemas:
-                    if schema in self._schema_map:
-                        logger.error(
-                            "{} schema ({}) mismatch detected - {} to {}"
-                            .format(self.name, schema, self._schema_map,
-                                    plugin))
-                        continue
-
-                    # Assign plugin
-                    self._schema_map[schema] = plugin
-
                 logger.trace(
                     '{} {} loaded in {:.6f}s'.format(
                         self.name, module_name, (time.time() - tl_start)))
@@ -365,67 +373,66 @@ class PluginManager(metaclass=Singleton):
             # end of _import_module()
             return
 
-        with self._lock:
-            for _path in paths:
-                path = os.path.abspath(os.path.expanduser(_path))
-                if (cache and path in self._paths_previously_scanned) \
-                        or not os.path.exists(path):
-                    # We're done as we've already scanned this
-                    continue
+        for _path in paths:
+            path = path_decode(_path)
+            if (cache and path in self._paths_previously_scanned) \
+                    or not os.path.exists(path):
+                # We're done as we've already scanned this
+                continue
 
-                # Store our path as a way of hashing it has been handled
-                self._paths_previously_scanned.add(path)
+            # Store our path as a way of hashing it has been handled
+            self._paths_previously_scanned.add(path)
 
-                if os.path.isdir(path) and not \
-                        os.path.isfile(os.path.join(path, '__init__.py')):
+            if os.path.isdir(path) and not \
+                    os.path.isfile(os.path.join(path, '__init__.py')):
 
-                    logger.debug('Scanning for custom plugins in: %s', path)
-                    for entry in os.listdir(path):
-                        re_match = module_re.match(entry)
-                        if not re_match:
-                            # keep going
-                            logger.trace('Plugin Scan: Ignoring %s', entry)
-                            continue
-
-                        new_path = os.path.join(path, entry)
-                        if os.path.isdir(new_path):
-                            # Update our path
-                            new_path = os.path.join(path, entry, '__init__.py')
-                            if not os.path.isfile(new_path):
-                                logger.trace(
-                                    'Plugin Scan: Ignoring %s',
-                                    os.path.join(path, entry))
-                                continue
-
-                        if not cache or \
-                                (cache and new_path not in
-                                 self._paths_previously_scanned):
-                            # Load our module
-                            _import_module(new_path)
-
-                            # Add our subdir path
-                            self._paths_previously_scanned.add(new_path)
-                else:
-                    if os.path.isdir(path):
-                        # This logic is safe to apply because we already
-                        # validated the directories state above; update our
-                        # path
-                        path = os.path.join(path, '__init__.py')
-                        if cache and path in self._paths_previously_scanned:
-                            continue
-
-                        self._paths_previously_scanned.add(path)
-
-                    # directly load as is
-                    re_match = module_re.match(os.path.basename(path))
-                    # must be a match and must have a .py extension
-                    if not re_match or not re_match.group(1):
+                logger.debug('Scanning for custom plugins in: %s', path)
+                for entry in os.listdir(path):
+                    re_match = module_re.match(entry)
+                    if not re_match:
                         # keep going
-                        logger.trace('Plugin Scan: Ignoring %s', path)
+                        logger.trace('Plugin Scan: Ignoring %s', entry)
                         continue
 
-                    # Load our module
-                    _import_module(path)
+                    new_path = os.path.join(path, entry)
+                    if os.path.isdir(new_path):
+                        # Update our path
+                        new_path = os.path.join(path, entry, '__init__.py')
+                        if not os.path.isfile(new_path):
+                            logger.trace(
+                                'Plugin Scan: Ignoring %s',
+                                os.path.join(path, entry))
+                            continue
+
+                    if not cache or \
+                            (cache and new_path not in
+                             self._paths_previously_scanned):
+                        # Load our module
+                        _import_module(new_path)
+
+                        # Add our subdir path
+                        self._paths_previously_scanned.add(new_path)
+            else:
+                if os.path.isdir(path):
+                    # This logic is safe to apply because we already
+                    # validated the directories state above; update our
+                    # path
+                    path = os.path.join(path, '__init__.py')
+                    if cache and path in self._paths_previously_scanned:
+                        continue
+
+                    self._paths_previously_scanned.add(path)
+
+                # directly load as is
+                re_match = module_re.match(os.path.basename(path))
+                # must be a match and must have a .py extension
+                if not re_match or not re_match.group(1):
+                    # keep going
+                    logger.trace('Plugin Scan: Ignoring %s', path)
+                    continue
+
+                # Load our module
+                _import_module(path)
 
         return None
 
